@@ -1,5 +1,5 @@
 from .aws import S3Location, LoadPartitions
-from .db import CachedAthenaQuery, CachedPep249Query, BaseCachedQuery, AthenaClient, SqlClient
+from .db import CachedAthenaQuery, CachedPep249Query, BaseCachedQuery, AthenaClient, SqlClient, CachedCTASQuery
 from .doggo import PandasDoggo, FileDoggo, CSVDoggo, DoggoFileSystem, DoggoLock, DoggoWait, DynamoDogLock
 from .log import log_to_stdout, PersistentFieldLogger, JSONLogger
 
@@ -9,5 +9,5 @@ __all__ = [
     'PandasDoggo', 'FileDoggo', 'CSVDoggo', 'DoggoFileSystem',
     'DoggoLock', 'DoggoWait',
     'log_to_stdout', 'PersistentFieldLogger', 'JSONLogger',
-    'AthenaClient', 'SqlClient', 'LoadPartitions'
+    'AthenaClient', 'SqlClient', 'LoadPartitions', 'CachedCTASQuery'
 ]
