@@ -882,7 +882,7 @@ class CachedCTASQuery(CachedAthenaQuery):
         """
         return {
             "database": params.get("ctas_athena_db"),
-            "table_name": output_file.split('/')[-2],
+            "table_name": re.sub(r'\W', '', output_file.split('/')[-2]),
             "external_location": output_file,
             "output_format": params.get("output_format"),
             "output_compression": params.get("output_compression"),
