@@ -7,25 +7,15 @@ import gzip
 import json
 import logging
 import os
-import re
 from datetime import datetime
-from io import BytesIO
-
+from typing import Optional, Union, Type, Iterator
 from newtools.aws import S3Location
-from newtools.optional_imports import AWSRetry
-from newtools.optional_imports import boto3
+from .fs import DoggoFileSystem, FileDoggo
+
 from newtools.optional_imports import pandas as pd
 
-
-def _gzip_wrapped(func):
-    def gzip_open(self, *args, **kwargs):
-        if self.compression == 'gzip':
-            f = func(self, *args, **kwargs)
-            self._to_close.append(f)
-            return gzip.open(f, self.mode)
-        return func(self, *args, **kwargs)
-
-    return gzip_open
+Path = Union[str, S3Location]
+DataFrame = Type['pd.DataFrame']
 
 
 class PandasDoggo:
@@ -88,6 +78,7 @@ class PandasDoggo:
         else:
             self.s3_client = None
 
+        self.dfs = DoggoFileSystem()
         self._logger = logger
 
     @staticmethod
@@ -271,118 +262,43 @@ class PandasDoggo:
 
         return paths
 
-
-class FileDoggo:
-    """
-
-He fetches the things you want. Tv remotes, slippers, newspapers, files. Mostly files though.
-
-.. code-block:: python
-
-    from newtools import FileDoggo
-    import boto3
-
-    path = 's3 or local path'
-    s3_client = boto3.Session().client('s3')
-    with FileDoggo(path, mode='rb', client=s3_client) as f:
-        f.read()
-
-This is written to treat local paths and s3 paths the same, returning a file like object for either.
-
-"""
-    _connection = None
-
-    def __init__(self, path, mode='rb', is_s3=None, client=None, compression=None, request_payer='bucketowner'):
+    def load_dir_iter(
+        self,
+        load_path: Path,
+        file_format: Optional[str] = None
+    ) -> Iterator[DataFrame]:
         """
-        Creates the class
+        Function to return a generator of the output under the given directory.
 
-        :param path: path to file to connect to. s3 or local path
-        :type path: str or S3Location
-        :param mode: opening mode for file, note s3 only allows 'rb' or 'wb'
-        :param is_s3: force Doggo to treat path as an S3Location, otherwise autodetects if beings with "s3://"
-        :type is_s3: bool
-        :param client: optional `boto3.Session().Client('s3')` instance to use for s3 operations
-        :param compression: None, 'gzip' or 'snappy'.
-        :param request_payer: either 'bucketowner' or 'requester' - who incurs the cost of said operation
+        :param load_path: the directory containing the data in S3
+        :param file_format: the format of the save files, e.g. PARQUET or CSV
         """
+        file_format = file_format.upper() if file_format else 'PARQUET'
 
-        self.buffer = BytesIO()
-        self.mode = mode
-
-        self.is_s3 = is_s3 or path.startswith('s3://')
-        if self.is_s3:
-            # allows you to specify s3 for paths that don't begin with s3:// for whatever reason you might want
-            self.path = S3Location(path)
+        if file_format == 'PARQUET':
+            load_func = self.load_parquet
+        elif file_format == 'CSV':
+            load_func = self.load_csv
         else:
-            self.path = path
+            raise NotImplementedError(
+                f"{self.__class__.__name__}.load_dir_iter only supports loading CSV or PARQUET files, not {file_format}"
+            )
 
-        self.request_payer = request_payer
+        for file_name in self.dfs.ls(location=load_path, recursive=True):
+            yield load_func(file_name)
 
-        self._client = client
-        if compression in [None, 'gzip', 'snappy']:
-            self.compression = compression
-        else:
-            raise NotImplementedError(f"Compression {compression} is not supported in FileDoggo")
-        self._to_close = []
+    def load_dir(
+        self,
+        load_path: Path,
+        file_format: Optional[str] = None
+    ) -> DataFrame:
+        """
+        Function to load in the CTAS output and return one unified dataframe.
 
-    @property
-    def connection(self):
-        if not self._connection:
-            self._connection = self._connect()
-        return self._connection
+        :param load_path: the directory containing the CTAS data in S3
+        :param file_format: the format of the save files, e.g. PARQUET or CSV
+        """
+        file_format = file_format.upper() if file_format else 'PARQUET'
+        return pd.concat(df for df in self.load_dir_iter(load_path, file_format))
 
-    @property
-    def client(self):
-        if not self._client:
-            self._client = boto3.Session().client('s3')
-        return self._client
 
-    def close(self):
-
-        if self._connection:
-            for c in [self._connection] + self._to_close:
-                if not c.closed:
-                    c.close()
-
-    def __enter__(self):
-        return self.connection
-
-    def __exit__(self, etype, value, traceback):
-        # ¯\_(ツ)_/¯
-        if self.is_s3 and 'w' in self.mode:
-            self._write_s3()
-
-        self.close()
-
-    @_gzip_wrapped
-    def _connect(self):
-
-        if self.is_s3:
-            if self.mode == 'rb':
-                return self._read_s3()
-            elif self.mode == 'wb':
-                # check s3 exists? meh, writing to a buffer is so fast i think the overhead isn't worth
-                return self.buffer  # write to this buffer and we'll upload later
-            else:
-                raise ValueError(f'mode {self.mode} is not supported')
-
-        else:
-
-            return open(self.path, self.mode)
-
-    @AWSRetry.backoff(tries=5, delay=1, backoff=1, added_exceptions=['404'])
-    def _read_s3(self):
-
-        self.client.download_fileobj(Bucket=self.path.bucket, Key=self.path.key, Fileobj=self.buffer,
-                                     ExtraArgs={'RequestPayer': self.request_payer})
-        self.buffer.seek(0)
-        return self.buffer
-
-    def _write_s3(self):
-
-        if self.compression == 'gzip':
-            self.connection.close()  # flushes writing to gzip
-
-        self.buffer.seek(0)
-        self.client.upload_fileobj(Bucket=self.path.bucket, Key=self.path.key, Fileobj=self.buffer,
-                                   ExtraArgs={'RequestPayer': self.request_payer})
