@@ -31,18 +31,145 @@ Cross-platform supports for reading, writing, copying, deleting, etc... files fr
        `^._,--'           / , , .)
                           `-._,-'
 """
-
 import shutil
 import os
+import gzip
+from io import BytesIO
 from glob import glob
 from newtools.optional_imports import s3fs, boto3, botocore
 from newtools.aws import S3Location
-from .doggo import FileDoggo
 from newtools.log import log_to_stdout
+from newtools.optional_imports import AWSRetry
+from newtools.optional_imports import boto3
 
 import fnmatch
 logger = log_to_stdout('Deductive - DoggoFileSystem')
 
+def _gzip_wrapped(func):
+    def gzip_open(self, *args, **kwargs):
+        if self.compression == 'gzip':
+            f = func(self, *args, **kwargs)
+            self._to_close.append(f)
+            return gzip.open(f, self.mode)
+        return func(self, *args, **kwargs)
+
+    return gzip_open
+
+class FileDoggo:
+    """
+
+He fetches the things you want. Tv remotes, slippers, newspapers, files. Mostly files though.
+
+.. code-block:: python
+
+    from newtools import FileDoggo
+    import boto3
+
+    path = 's3 or local path'
+    s3_client = boto3.Session().client('s3')
+    with FileDoggo(path, mode='rb', client=s3_client) as f:
+        f.read()
+
+This is written to treat local paths and s3 paths the same, returning a file like object for either.
+
+"""
+    _connection = None
+
+    def __init__(self, path, mode='rb', is_s3=None, client=None, compression=None, request_payer='bucketowner'):
+        """
+        Creates the class
+
+        :param path: path to file to connect to. s3 or local path
+        :type path: str or S3Location
+        :param mode: opening mode for file, note s3 only allows 'rb' or 'wb'
+        :param is_s3: force Doggo to treat path as an S3Location, otherwise autodetects if beings with "s3://"
+        :type is_s3: bool
+        :param client: optional `boto3.Session().Client('s3')` instance to use for s3 operations
+        :param compression: None, 'gzip' or 'snappy'.
+        :param request_payer: either 'bucketowner' or 'requester' - who incurs the cost of said operation
+        """
+
+        self.buffer = BytesIO()
+        self.mode = mode
+
+        self.is_s3 = is_s3 or path.startswith('s3://')
+        if self.is_s3:
+            # allows you to specify s3 for paths that don't begin with s3:// for whatever reason you might want
+            self.path = S3Location(path)
+        else:
+            self.path = path
+
+        self.request_payer = request_payer
+
+        self._client = client
+        if compression in [None, 'gzip', 'snappy']:
+            self.compression = compression
+        else:
+            raise NotImplementedError(f"Compression {compression} is not supported in FileDoggo")
+        self._to_close = []
+
+    @property
+    def connection(self):
+        if not self._connection:
+            self._connection = self._connect()
+        return self._connection
+
+    @property
+    def client(self):
+        if not self._client:
+            self._client = boto3.Session().client('s3')
+        return self._client
+
+    def close(self):
+
+        if self._connection:
+            for c in [self._connection] + self._to_close:
+                if not c.closed:
+                    c.close()
+
+    def __enter__(self):
+        return self.connection
+
+    def __exit__(self, etype, value, traceback):
+        # ¯\_(ツ)_/¯
+        if self.is_s3 and 'w' in self.mode:
+            self._write_s3()
+
+        self.close()
+
+    @_gzip_wrapped
+    def _connect(self):
+
+        if self.is_s3:
+            if self.mode == 'rb':
+                return self._read_s3()
+            elif self.mode == 'wb':
+                # check s3 exists? meh, writing to a buffer is so fast i think the overhead isn't worth
+                return self.buffer  # write to this buffer and we'll upload later
+            else:
+                raise ValueError(f'mode {self.mode} is not supported')
+
+        else:
+
+            return open(self.path, self.mode)
+
+    @AWSRetry.backoff(tries=5, delay=1, backoff=1, added_exceptions=['404'])
+    def _read_s3(self):
+
+        self.client.download_fileobj(Bucket=self.path.bucket, Key=self.path.key, Fileobj=self.buffer,
+                                     ExtraArgs={'RequestPayer': self.request_payer})
+        self.buffer.seek(0)
+        return self.buffer
+
+    def _write_s3(self):
+
+        if self.compression == 'gzip':
+            self.connection.close()  # flushes writing to gzip
+
+        self.buffer.seek(0)
+        self.client.upload_fileobj(Bucket=self.path.bucket, Key=self.path.key, Fileobj=self.buffer,
+                                   ExtraArgs={'RequestPayer': self.request_payer})
+
 
 class DoggoFileSystem:
     """
